@@ -1,3 +1,3 @@
-API_VERSION=3
-API_HOST='api.lumino.so'
+API_VERSION=4
+API_HOST='api.luminoso.com'
 URL_BASE='https://%s/v%d' % (API_HOST, API_VERSION)
